package cmb.pb.shield.channelnumber;

import java.io.File;

import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.util.HashMap;
import java.util.Map;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;
import cmb.pb.shield.channelnumber.reader.ApkUtil;
import cmb.pb.shield.channelnumber.reader.PayloadReader;
import cmb.pb.shield.channelnumber.util.Base64;

/*验证渠道包签名，并取出渠道信息*/

public class ChannelNumber {
	// 使用的私钥
	private static final String PUBLIC = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCkPmTAyVMp1ONNn+NH8QboYCD/HQTIsymrZVn6"
			+ "d2JyAO5dcvf8emw8qtzG/wDTkPvnybWsz5+4p3uUGNGYB3pL8CExWCHQ893eNBbdDdiTeV2cglSV"
			+ "ra3lW5FtNh/kZoiyL08qoioAq6fFZPFcaPiX247josRTtjUuGvNOoQYJjwIDAQAB";
	private Context context;
	private String signInfo = null;
	private String channelNum = null;
	private static final String LOGTAG = "ChannelNumber";
	private static final String EXCEPTION = "Exception";

	public ChannelNumber(Context context) {
		this.context = context;

		Map<String, String> channel = getChannelInfo(context);
		if (channel != null) {
			signInfo = channel.get("signInfoPart1") + channel.get("signInfoPart2");
			channelNum = channel.get("channelName");
			if (!check(channelNum, signInfo)) {
				channelNum = null;
			}
		}

	}

	/* 获取公钥 */
	private static PublicKey getPublicKey() {
		PublicKey publicKey = null;
		try {// 解密密钥
			byte[] keyBytes;
			keyBytes = Base64.decodeBuffer(PUBLIC);

			// 解密公钥
			X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
			KeyFactory keyFactory = KeyFactory.getInstance("RSA");
			publicKey = keyFactory.generatePublic(keySpec);

		} catch (NoSuchAlgorithmException e) {
			Log.e(LOGTAG, "NoSuchAlgorithmException", e);
		} catch (InvalidKeySpecException e) {
			Log.e(LOGTAG, "InvalidKeySpecException", e);
		}

		return publicKey;
	}

	/* 验证签名 */
	private static boolean check(String channelName, String signInfo) {
		boolean result = false;
		try {
			byte[] channelByte = channelName.getBytes();
			byte[] signInfoByte = Base64.decode(signInfo.getBytes());
			// 读取公钥
			PublicKey publicKey = getPublicKey();

			Signature signature = Signature.getInstance("SHA1WithRSA");
			signature.initVerify(publicKey);

			signature.update(channelByte);

			result = signature.verify(signInfoByte);

		} catch (Exception e) {
			Log.e(LOGTAG, EXCEPTION, e);
		}

		return result;
	}



	/* 获取需要的标签的对应值 */
	public String getChannelInfo(String channelName) {
		return channelNum;
	}

	/**
	 * get channel info (include channle & extraInfo)
	 *
	 * @param context context
	 * @return channel info
	 */
	@Nullable
	public static Map<String, String> getChannelInfo(@NonNull final Context context) {
		Map<String, String> channel = null;
		try {
			final String apkPath = getApkPath(context);
			if (TextUtils.isEmpty(apkPath)) {
				return null;
			}
			File apkFile = new File(apkPath);
			String payload_channelName = PayloadReader.getString(apkFile, ApkUtil.APK_CHANNEL_BLOCK_ID);
			String payload_signInfoPart1 = PayloadReader.getString(apkFile, ApkUtil.APK_CHANNEL_SIGN_INFO_BLOCK1_ID);
			String payload_signInfoPart2 = PayloadReader.getString(apkFile, ApkUtil.APK_CHANNEL_SIGN_INFO_BLOCK2_ID);
			if(null!=payload_channelName || null!=payload_signInfoPart1 || null!=payload_signInfoPart2) {
				channel = new HashMap<String, String>();
				channel.put("channelName", payload_channelName);
				channel.put("signInfoPart1", payload_signInfoPart1);
				channel.put("signInfoPart2", payload_signInfoPart2);
			}
		}catch (Exception e) {
			Log.e(LOGTAG, EXCEPTION, e);
		}
		return channel;
	}

	@Nullable
	private static String getApkPath(@NonNull final Context context) {
		String apkPath = null;
		try {
			final ApplicationInfo applicationInfo = context.getApplicationInfo();
			if (applicationInfo == null) {
				return null;
			}
			apkPath = applicationInfo.sourceDir;
		} catch (Exception e) {
			Log.e(LOGTAG, EXCEPTION, e);
		}
		return apkPath;
	}

}
