package cmb.pb.shield.channelnumber.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import android.util.Log;


public class Base64 {
	private static final String EXCEPTION = "ASCII is not supported!";
	private static final String CODE = "ASCII";
	private static final String LOGTAG = "Base64";
	
	private Base64() {}
	
	static class MyException extends RuntimeException{
		public MyException(String exception,Throwable e) {
			super(exception,e);
		}
	}
	
    public static String decode(String str){
        byte[] bytes;
        try {
            bytes = str.getBytes(CODE);
        } catch (UnsupportedEncodingException e) {
            throw new MyException(EXCEPTION, e);
        }
        byte[] decoded = decode(bytes);
        return new String(decoded);
    }
    
    public static byte[] decodeBuffer(String str){
        byte[] bytes;
        try {
            bytes = str.getBytes(CODE);
        } catch (UnsupportedEncodingException e) {
            throw new MyException(EXCEPTION, e);
        }
        byte[] decoded = decode(bytes);
        return decoded;
    }

    public static byte[] decode(byte[] bytes){
        ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            decode(inputStream, outputStream);
        } catch (IOException e) {
            throw new MyException(EXCEPTION, e);
        } finally {
            try {
                inputStream.close();
            } catch (Exception e) {
            	Log.e(LOGTAG, "InputStream Close Error", e);
            }
            try {
                outputStream.close();
            } catch (Exception e) {
            	Log.e(LOGTAG, "OutputStream Close Error", e);
            }
        }
        return outputStream.toByteArray();
    }

    public static void decode(InputStream inputStream, OutputStream outputStream)
            throws IOException {
        copy(new Base64InputStream(inputStream), outputStream);
    }

    private static void copy(InputStream inputStream, OutputStream outputStream)
            throws IOException {
        // 1KB buffer
        byte[] b = new byte[1024];
        int len;
        while ((len = inputStream.read(b)) != -1) {
            outputStream.write(b, 0, len);
        }
    }

}
